import os
import glob
import json
import base64
import logging

from avrs.simconfig_util import *

def get_payload(cfg_object, payload_name):
    for p in cfg_object['payloads']:
        if p['typeName'] == payload_name:
            return p
    return None

def clear_autospawns():
    logger = logging.getLogger('avrs')
    # also need to edit the yas_marina_env.json to have autospawn for this config
    sim_path = os.environ.get('AVRS_INSTALL_PATH', 
        os.path.join(os.environ['HOME'], 'autoverse-linux'))
    sim_saved = os.path.join(sim_path, 'Linux', 'Autoverse', 'Saved')
    logger.info('clearing autospawns from sim saved at {}'.format(sim_saved))

    cfg_files = SimConfigFiles(sim_saved)

    cfg_ok, msg = cfg_files.validate()
    if not cfg_ok:
        print(msg)
        return

    # also remove all configs from Objects dir
    files = glob.glob(os.path.join(sim_saved, 'Objects/*'))
    for f in files:
        logger.info('removing object config: {}'.format(f))
        os.remove(f)


    cfg_files.files['yas']['autoSpawnObjects'] = []
    cfg_files.files['main']['objectTemplatePaths'] = []
    cfg_files.save()

def register_received_vehicle(team_name, slot, cfg_data, bsu_vcan, kistler_vcan, badenia_vcan):
    logger = logging.getLogger('avrs')
    logger.info('registering received vehicle in slot {} for team {}'.format(slot, team_name))

    cfg_string = base64.b64decode(cfg_data)
    cfg_object = json.loads(cfg_string.decode('utf-8'))

    # ensure perception is disabled
    eav24 = get_payload(cfg_object, 'Eav24Initializer')
    if eav24 is None:
        return (False, 'no eav24 payload found')

    logger.info('disabling perception for received vehicle config')
    eav24['body']['bLidarEnabled'] = False
    eav24['body']['bCameraEnabled'] = False
    eav24['body']['bRadarEnabled'] = False
    eav24['body']['bPublishGroundTruth'] = False
    eav24['body']['bPublishInputs'] = False

    logger.info('disabling hud for received vehicle config')
    eav24['body']['bHudEnabled'] = False

    logger.info('setting bsu vcan to: {}, kistler to: {}, and badenia to: {}'.format(
        bsu_vcan, kistler_vcan, badenia_vcan))
    eav24['body']['bsuCanName'] = bsu_vcan
    eav24['body']['kistlerCanName'] = kistler_vcan
    eav24['body']['badeniaCanName'] = badenia_vcan

    ros2 = get_payload(cfg_object, 'Ros2')
    if ros2 is None:
        logger.info('no ros2 payload found. adding with domain id {}'.format(slot))
        # need to add a ros2 payload for domain id
        cfg_object['payloads'].append({
            'typeName': 'Ros2',
            'body': {
                'domainId': slot
            }
        })
    else:
        logger.info('found Ros2 payload OK'.format())
        logger.info('setting ros2 domain id to {}'.format(slot))
        ros2['body']['domainId'] = slot

    # auto add display widget
    wtc = get_payload(cfg_object, 'WorldTextComponent')
    if wtc is None:
        logger.info('no WorldTextComponent payload found. adding')
        cfg_object['payloads'].append({
            'typeName': 'WorldTextComponent',
            'body': {

            }
        })

    # do not allow default object name (collision)
    if cfg_object['name'] == 'eav24':
        logger.info('setting vehicle name from default to team name: {}'.format(team_name))
        cfg_object['name'] = team_name
    object_spec_name = 'eav24_{}'.format(team_name)
    cfg_object['specName'] = object_spec_name

    # also need to edit the yas_marina_env.json to have autospawn for this config
    sim_path = os.environ.get('AVRS_INSTALL_PATH', 
        os.path.join(os.environ['HOME'], 'autoverse-linux'))
    sim_saved = os.path.join(sim_path, 'Linux', 'Autoverse', 'Saved')
    cfg_files = SimConfigFiles(sim_saved)

    cfg_ok, msg = cfg_files.validate()
    if not cfg_ok:
        print(msg)
        return

    start_landmark = 'MvStart{}'.format(slot)

    entry_exists = False
    for i in cfg_files.files['yas']['autoSpawnObjects']:
        if 'objectSpec' in i and i['objectSpec'] == cfg_object['specName']:
            entry_exists = True
            logger.info('config is already in auto spawn list')

    if not entry_exists:
        logger.info('config is not in auto spawn list. adding')
        cfg_files.files['yas']['autoSpawnObjects'].append({
                'bShouldSpawn': True,
                'objectType': 'Eav24',
                'objectSpec': cfg_object['specName'],
                'bSpawnAtLandmark': True,
                'spawnLandmarkName': start_landmark
            })

    # also need to add the object template to main sim config

    new_cfg_name = 'eav24_{}.json'.format(object_spec_name)
    cfg_files.files['main']['objectTemplatePaths'].append(os.path.join('Objects', new_cfg_name))
    logger.info('saving config file: {}'.format(new_cfg_name))
    cfg_files.save()

    target_path = os.path.join(sim_saved, 'Objects', new_cfg_name)
    with open(target_path, 'w', encoding='utf-8') as f:
        json.dump(cfg_object, f, ensure_ascii=False, indent=4)

def prepare_vehicle_cfg(cfg_path):
    logger = logging.getLogger('avrs')
    #print('preparing config for transmission: {}'.format(cfg_path))

    if not os.path.isfile(cfg_path):
        return (False, '{} is not a valid file'.format(cfg_path), None, None, None)

    cfg_object = {}
    with open(cfg_path, 'r', encoding='utf-8') as f:
        cfg_object = json.load(f)


    # obtain desired CAN names to start cannelloni
    eav24 = get_payload(cfg_object, 'Eav24Initializer')
    if eav24 is None:
        return (False, 'no eav24 payload found', None, None, None)

    bsu_vcan = eav24['body']['bsuCanName']
    kistler_vcan = eav24['body']['kistlerCanName']
    badenia_vcan = eav24['body']['badeniaCanName']

    logger.info('detected vcan names from sent config: bsu {}, kistler {}, badenia {}'.format(
        bsu_vcan, kistler_vcan, badenia_vcan))

    cfg_data = base64.b64encode(json.dumps(cfg_object).encode('utf-8')).decode('utf-8')
    return (True, cfg_data, bsu_vcan, kistler_vcan, badenia_vcan)

